/*
 *	Copyright © 2015 Matthieu DUVAL, Rudy DUCHE
 *
 *	This file is part of RizzleHash.
 *
 *	RizzleHash is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	RizzleHash is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with RizzleHash.  If not, see <http://www.gnu.org/licenses/>
 */
#include "../include/RH_Constants.h"

RH_Constants::RH_Constants()
{
    //applicationName = "RizzleHash V0.8 - rc1";
}

string RH_Constants::getOSName()
{
    #ifdef _WIN32
    return "Windows 32-bit";
    #elif _WIN64
    return "Windows 64 bit";
    #elif __APPLE__ ||__MACH__
    return "degage connard!";
    #elif __linux__
    return "Linux";
    #elif __FreeBSD__
    return "FreeBSD";
    #elif __unix || __unix__
    return "Unix";
    #else
    return "Unknown";
    #endif // _WIN32
}

int RH_Constants::getNumberOfCore()
{
	CPUID *cpuID = new CPUID(0x0B,0x01);
	int nbCore = cpuID->EBX();
	nbCore = nbCore & 0x0000FFFF;

    delete cpuID;
    return nbCore;
}

string RH_Constants::getProcessorVendor()
{
    CPUID *cpuID = new CPUID(0x00,0x00);
	string vendor;
	vendor += string((const char*)&cpuID->EBX(),4);
	vendor += string((const char*)&cpuID->EDX(),4);
	vendor += string((const char*)&cpuID->ECX(),4);
    delete cpuID;
    return vendor;
}

string RH_Constants::getProcessorName()
{
    CPUID *cpuID = new CPUID(0x80000000,0x00);
    string name;
    if( cpuID->EAX() >= 0x80000004)
    {
        cpuID = new CPUID(0x80000002,0x00);
        int32_t value[12];
        value[0] = cpuID->EAX();
        value[1] = cpuID->EBX();
        value[2] = cpuID->ECX();
        value[3] = cpuID->EDX();

        cpuID = new CPUID(0x80000003,0x00);
        value[4] = cpuID->EAX();
        value[5] = cpuID->EBX();
        value[6] = cpuID->ECX();
        value[7] = cpuID->EDX();

        cpuID = new CPUID(0x80000004,0x00);
        value[8] = cpuID->EAX();
        value[9] = cpuID->EBX();
        value[10] = cpuID->ECX();
        value[11] = cpuID->EDX();

        int taille = 0;
        bool eosFound = false;
        for( unsigned int i = 0; i< sizeof(value) ; i++)
        {
            char bytes[] ={0x00,0x00,0x00,0x00};
            bytes[0] = value[i] &0x000000FF;
            bytes[1] = (value[i] >> 8 )&0x000000FF;
            bytes[2] = (value[i] >> 16)&0x000000FF;
            bytes[3] = (value[i] >> 24)&0x000000FF;
            for(unsigned int j=0;j< sizeof(bytes);j++)
            {

                if(bytes[j] == 0x00)
                {
                    eosFound = true;
                }
                if(!eosFound)   //si on a trouvé le caractère de fin de chaine,
                                //on arrete d'augmenter le taille de la chaine
                {
                    taille++;
                }

            }
        }
        /**
         *  Suppression des espace en double
         */
        list<char> bytes = trimProcName(value,taille);
        char value2[bytes.size()];
        for (list<char>::iterator i = bytes.begin(); i != bytes.end(); i++)
        {
            value2[distance(bytes.begin(),i)] = *i;
        }

        //name = string((const char*)&value,taille);
        name = string((const char*)&value2,bytes.size());
    }
    else
    {
        name = "not supported";
    }
    //name = "not supported";
	delete cpuID;
    return name;
}

list<char> RH_Constants::trimProcName(int32_t * value, int taille)
{
    list<char> bytes;
    int j = 0;
    for( unsigned int i = 0; i< 48; i++)
    {
        /*
         *  partie 1
         */
        if( j==0 && (value[i] &0x000000FF) != 0x20)//on suprime le première espace
        {
            bytes.push_back(value[i] &0x000000FF);
        }
        if(i >0 && j < taille &&!((((value[i-1] >> 24 )&0x000000FF) == 0x20)&& ((value[i] &0x000000FF) == 0x20)))
        {
             bytes.push_back(value[i] &0x000000FF);
        }
        j++;

        if(j < taille && !(((value[i] &0x000000FF) == 0x20) && ((value[i]>> 8) &0x000000FF) == 0x20))
        {
             bytes.push_back((value[i] >> 8 )&0x000000FF);
        }
        j++;

        if(j < taille && !((((value[i] >> 8)&0x000000FF) == 0x20) && ((value[i]>> 16) &0x000000FF) == 0x20))
        {
             bytes.push_back((value[i] >> 16 )&0x000000FF);
        }
        j++;

        if(j < taille && !((((value[i] >> 16)&0x000000FF) == 0x20) && ((value[i]>> 24) &0x000000FF) == 0x20))
        {
             bytes.push_back((value[i] >> 24 )&0x000000FF);
        }
        j++;
    }
    return bytes;
}
