/*
 *	Copyright © 2015 Matthieu DUVAL, Rudy DUCHE
 *
 *	This file is part of RizzleHash.
 *
 *	RizzleHash is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	RizzleHash is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with RizzleHash.  If not, see <http://www.gnu.org/licenses/>
 */
#include "../include/RH_Encryption.h"

RH_Encryption::RH_Encryption(){
    //ctor
}

function<string (string)> RH_Encryption::md5(){
    function<string (string)> methode = [] (string value){
        const unsigned char* stringValue = (unsigned char *) value.c_str();
        MD5_CTX md5;
        MD5_Init(&md5);
        MD5_Update(&md5,(const unsigned char*)stringValue, value.length());
        unsigned char buffer[MD5_DIGEST_LENGTH];
        MD5_Final(buffer, &md5);
        return  bytesArrayToStrinctString(MD5_DIGEST_LENGTH,buffer);
    };
    return methode;
}

function<string (string)> RH_Encryption::sha1(){
    return [](string value){
        const unsigned char* stringValue = (unsigned char *) value.c_str();
        unsigned char buffer[SHA_DIGEST_LENGTH];
        SHA(stringValue,value.length(),buffer);
        return bytesArrayToStrinctString(SHA_DIGEST_LENGTH,buffer);
    };
}

function<string (string)> RH_Encryption::sha224(){
    return [](string value){
        const unsigned char* stringValue = (unsigned char *) value.c_str();
        unsigned char buffer[SHA224_DIGEST_LENGTH];
        SHA224(stringValue,value.length(),buffer);
        return bytesArrayToStrinctString(SHA224_DIGEST_LENGTH,buffer);
    };
}

function<string (string)> RH_Encryption::sha384(){
    return [](string value){
        const unsigned char* stringValue = (unsigned char *) value.c_str();
        unsigned char buffer[SHA384_DIGEST_LENGTH];
        SHA384(stringValue,value.length(),buffer);
        return bytesArrayToStrinctString(SHA384_DIGEST_LENGTH,buffer);
    };
}

function<string (string)> RH_Encryption::sha256(){
    return [](string value){
        const unsigned char* stringValue = (unsigned char *) value.c_str();
        unsigned char buffer[SHA256_DIGEST_LENGTH];
        SHA256(stringValue,value.length(),buffer);
        return bytesArrayToStrinctString(SHA256_DIGEST_LENGTH,buffer);
    };
}

function<string (string)> RH_Encryption::sha512( ){
    return [](string value){
        const unsigned char* stringValue = (unsigned char *) value.c_str();
        unsigned char buffer[SHA512_DIGEST_LENGTH];
        SHA256(stringValue,value.length(),buffer);
        return bytesArrayToStrinctString(SHA512_DIGEST_LENGTH,buffer);
    };
}

function<string (string)> RH_Encryption::none(){
    return [](string value){
        return value;
    };
}

function<string(string)> RH_Encryption::getFunctionHashLambda(string hashMethode) {
    function<string(string)> resultMethode;

    if (hashMethode == "md5")           resultMethode = md5();
    else if(hashMethode == "sha1")      resultMethode = sha1();
    else if(hashMethode == "sha224")    resultMethode = sha224();
    else if(hashMethode == "sha384")    resultMethode = sha384();
    else if(hashMethode == "sha256")    resultMethode = sha256();
    else if(hashMethode == "sha512")    resultMethode = sha512();
    else                                resultMethode = none();

    return resultMethode;
}


/*function<string (string)> RH_Encryption::hashForLambda(function<string(string)> functionLambda, string value){
    return functionLambda(value);
}*/

string RH_Encryption::bytesArrayToStrinctString(uint8_t lenght, unsigned char * bytes) {
    char resString[2];
    string result;
    for (uint8_t i = 0; i < lenght; i++){
        sprintf(resString,"%02x", bytes[i]);
        result.append(resString);
    }
    return result;
}


RH_Encryption::~RH_Encryption(){
    //dtor
}
