/*
 *	Copyright © 2015 Matthieu DUVAL, Rudy DUCHE
 *
 *	This file is part of RizzleHash.
 *
 *	RizzleHash is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	RizzleHash is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with RizzleHash.  If not, see <http://www.gnu.org/licenses/>
 */

#include "../include/RH_GUI.h"

GUI::GUI()
{
	this->guiTime = time(NULL);
	this->headerLines = this->prepareHeader();
}

GUI::~GUI()
{

}

void GUI::displayHeader()
{
	this->clearScreen();
	for(list<string>::iterator i = this->headerLines.begin(); i != this->headerLines.end(); i++)
	{
		cout << *i << endl;
	}
}

void GUI::displayProcessPerfs(string host, string processName, int WordsSeconds, string currentWord)
{
    // Ce bloc devrat disparaitre
    stringstream ss;
    ss.imbue(locale(""));
    ss << fixed << WordsSeconds;
    //cout << "Host: " <<  host << endl;
    cout << "\t" << processName << ": \t" << ss.str() << "/s\tcurrentWord: [ " <<  currentWord << " ]" << endl;
}

void GUI::displayProcessPerfsList(string host, list<string> processName, list<int>WordsSeconds, list<string> currentWord)
{
	cout << "Not yet implemented " << endl;
}

void GUI::displayTotalPerfs(string host,int WordsSeconds)
{
	cout << "Not yet implemented " << endl;
}

void GUI::displayError(string host,string errorCode,string errorMessage)
{
	cout << "Not yet implemented " << endl;
}

void GUI::displayMessage(string host,string message)
{
	cout << "Not yet implemented " << endl;
}

void GUI::displayResult(string value,time_t secondes)
{
	cout << "Not yet implemented " << endl;
}

void GUI::displayCopyright()
{
	list<string> lines;
	list<string> info;
	int lenGui = 0;
	info.push_back("");
	info.push_back(APPLICATION_NAME + " Copyright (c) " + APPLICATION_YEAR + " " +  APPLICATION_AUTHORS);
	info.push_back("This program comes with ABSOLUTELY NO WARRANTY");//#; for details type `show w'.")
	info.push_back("This is free software, and you are welcome to redistribute it");
	info.push_back("under certain conditions. Please read the COPYING and README");//; type `show c' for details.");
	info.push_back("");

	for(list<string>::iterator i = info.begin(); i != info.end(); i++)
	{
		if ((*i).size() > lenGui)
		{
			lenGui = (*i).size();
		}

	}
	lines.push_back(this->separationLineCreator("*","*","*",lenGui+2));
	for (list<string>::iterator i = info.begin(); i != info.end(); i++)
	{
		lines.push_back(this->leftAlignLine("* "," *"," ",lenGui,*i));
	}
	lines.push_back(this->separationLineCreator("*","*","*",lenGui+2));
	this->clearScreen();
	for (list<string>::iterator i = lines.begin(); i != lines.end(); i++)
	{
		cout << *i << endl;
	}
	cout << endl << "press any key to continue..." << endl;
	cin.get();
}

void GUI::displayShortHelp()
{
    cout << endl;
    cout << "Usage : " << endl;
    list<string> leftTab;
    list<string> rightTab;

    leftTab.push_back("\t"+PARAMS_CONSTANTS::LONG_PARAMS::HELP +", "+ PARAMS_CONSTANTS::SHORT_PARAMS::HELP);
    leftTab.push_back("\t"+PARAMS_CONSTANTS::LONG_PARAMS::HASH+"=[passwordSearch]");
    leftTab.push_back("\t"+PARAMS_CONSTANTS::LONG_PARAMS::VERBOSE+", "+PARAMS_CONSTANTS::SHORT_PARAMS::VERBOSE);
    leftTab.push_back("\t"+PARAMS_CONSTANTS::LONG_PARAMS::MODE+", " +PARAMS_CONSTANTS::SHORT_PARAMS::MODE);

    rightTab.push_back("Display help message");
    rightTab.push_back("Set hash to search");
    rightTab.push_back("Set verbose mode");
    rightTab.push_back("Set application mode");


    int maxLenLeft = 0;
    for (list<string>::iterator i = leftTab.begin(); i != leftTab.end(); i++)
    {
        if((*i).length() > maxLenLeft)
        {
            maxLenLeft = (*i).size();
        }
    }

    list<string>::iterator j = rightTab.begin();

    for (list<string>::iterator i = leftTab.begin(); i != leftTab.end(); i++)
    {
        cout << this->tableLine(*i,*j," ",maxLenLeft+1) << endl;
        j++;
    }
}

void GUI::displayFullHelp()
{
	cout << "Not yet implemented " << endl;
}

void GUI::displayVersion()
{
	cout << "Not yet implemented " << endl;
}

int GUI::getInputInt(string question)
{
	cout << "Not yet implemented " << endl;
	return 0;
}

string GUI::getInputString(string question)
{
	cout << "Not yet implemented " << endl;
	return NULL;
}

int GUI::displayMenuInt(string title,list<string> entryList, string margin)
{
	cout << "Not yet implemented " << endl;
	return 0;
}

string GUI::displayMenuString(string title,list<string> entryList, string margin)
{
	cout << "Not yet implemented " << endl;
	return NULL;
}

void GUI::clearScreen()
{
	#if defined(__unix__) || defined(__unix) || defined(unix) || defined(__linux__)
		system("clear");
	#else
		system("cls");
	#endif // defined

}

void GUI::setLocale()
{
	cout << "Not yet implemented " << endl;
}

list<string> GUI::prepareHeader()
{
	const int INFO_SIZE = 4;
	const int LINES_SIZE = 8;
	list<string> info;
	list<string> lines;
	int lenGui = APPLICATION_NAME.size();

    // on convertis le in en string
    stringstream ss;
    ss << NB_CORE;
    string nbCore = ss.str();

    struct tm * timeinfo;
    timeinfo = localtime(&this->guiTime);
    //strftime(buffer,80,"%I:%M",timeinfo)
    //string time =  string(asctime(timeinfo));
    //trim(time);
    string date = to_string(timeinfo->tm_mday) + "/" +to_string(timeinfo->tm_mon) +"/" + to_string(timeinfo->tm_year+1900);
    string hour = to_string(timeinfo->tm_hour) + ":" +to_string(timeinfo->tm_min) +":" + to_string(timeinfo->tm_sec);
	info.push_back("System family : " + OS );
	info.push_back("Processor vendor: " + PROCESSOR_VENDOR);
	info.push_back("Processor name : " + PROCESSOR_NAME);
	info.push_back("Number of core : " + nbCore);
	info.push_back("App started at : " + date + " " + hour);

	for(list<string>::iterator i = info.begin(); i!=info.end();i++)	{
		if((*i).size() > lenGui)		{
			lenGui = (*i).size();
		}
	}
	lines.push_back(this->separationLineCreator("+", "+","-", lenGui+2));
	lines.push_back(this->centredLine("| "," |"," ",lenGui,APPLICATION_NAME));
	lines.push_back(this->separationLineCreator("+", "+","-", lenGui+2));
	for(list<string>::iterator i = info.begin(); i!=info.end();i++)
	{
		lines.push_back(this->leftAlignLine("| "," |"," ",lenGui,*i));
	}
	lines.push_back(this->separationLineCreator("+", "+","-", lenGui+2));
	return lines;
}

string GUI::separationLineCreator(string leftBroder, string rightBorder, string separator, int lenGui)
{
	string separationLine = leftBroder;
	for(int i=0;i<lenGui;i++)	{
		separationLine += separator;
	}
	separationLine += rightBorder;
	return separationLine;
}

string GUI::centredLine(string leftBroder, string rightBorder, string offsetChar, int lenGui,string value)
{
	int lenSpace = (lenGui -value.size())/2;
	int reste = (lenGui -value.size())%2;
	int lenSpaceLeft = lenSpace;
	int lenSpaceRight = lenSpace + reste;
	string spaceLeft;
	string spaceRight;
	for(int i = 0; i< lenSpaceLeft; i++)	{
		spaceLeft += offsetChar;
	}

	for(int i = 0; i< lenSpaceRight; i++)	{
		spaceRight += offsetChar;
	}
	string centeredLine = leftBroder + spaceLeft + value + spaceRight + rightBorder;
	return centeredLine;
}

string GUI::tableLine(string value1, string value2, string offsetChar, int lenMaxValue)
{
	string tableLine = "";
    string  spaceSeparator = "";
    for(int i=0;i<lenMaxValue - value1.length();i++)    {
        spaceSeparator += offsetChar;
    }
    tableLine = value1 + spaceSeparator + value2;
    return tableLine;
}

string GUI::leftAlignLine(string leftBroder, string rightBorder, string offsetChar, int lenGui,string value)
{
	int lenSpaceRight =  lenGui - value.size();
	string spaceRight ="";
	for( int i=0;i<lenSpaceRight;i++)	{
		spaceRight += offsetChar;
	}
	string leftAlignLine = leftBroder + value + spaceRight + rightBorder;
	return leftAlignLine;
}
